<?php

declare(strict_types=1);

namespace mirzaev\huesos;

// Files of the project
use mirzaev\huesos\controllers\core as controller,
	mirzaev\huesos\models\core as model,
	mirzaev\huesos\models\cart,
	mirzaev\huesos\models\telegram;

// Framework for Telegram
use Zanzara\Zanzara as zanzara,
	Zanzara\Context as context,
	Zanzara\Config as config;

// Framework for ArangoDB
use mirzaev\arangodb\document;

ini_set('error_reporting', E_ALL ^ E_DEPRECATED);
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);

// Версия робота
define('ROBOT_VERSION', '1.0.0');

// Путь до настроек
define('SETTINGS', __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'settings');

// Путь до хранилища
define('STORAGE', __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'storage');

// Файл в формате xlsx с примером excel-документа для импорта каталога
define('CATALOG_EXAMPLE', STORAGE . DIRECTORY_SEPARATOR . 'example.xlsx');

// Файл 
define('GREETING_VIDEO', STORAGE . DIRECTORY_SEPARATOR . 'greeting.mp4');

// Файл в формате xlsx для импорта каталога
define('CATALOG_IMPORT', STORAGE . DIRECTORY_SEPARATOR . 'import.xlsx');

/**
 * Ключ чат-робота Telegram 
 * @deprecated
 */
define('KEY', require(SETTINGS . DIRECTORY_SEPARATOR . 'telegram.php'));
define('TELEGRAM_KEY', require(SETTINGS . DIRECTORY_SEPARATOR . 'telegram.php'));

// Initialize dependencies
require __DIR__ . DIRECTORY_SEPARATOR
	. '..' . DIRECTORY_SEPARATOR
	. '..' . DIRECTORY_SEPARATOR
	. '..' . DIRECTORY_SEPARATOR
	. '..' . DIRECTORY_SEPARATOR
	. 'vendor' . DIRECTORY_SEPARATOR
	. 'autoload.php';

// Инициализация ядра контроллеров MINIMAL
/* new controller(new core, false); */

// Инициализация ядра моделей MINIMAL
new model(true);

$config = new config();
$config->setParseMode(config::PARSE_MODE_MARKDOWN);
$config->useReactFileSystem(true);

$robot = new zanzara(TELEGRAM_KEY, $config);

$robot->onUpdate(function (context $context): void {
	// Initializing the message
	$message = $context->getMessage();

	// Initializing the "web app" data
	$app = $message?->getWebAppData();

	if (!empty($app)) {
		// Initialized the "web app" data

		// Initializing request from "web app" data
		$request = json_decode($app->getData(), false, 10);

		if ($request->type === 'cart_share') {
			// Cart attaching

			// Attaching cart to the Telegram account
			telegram::cart_attach($context, $request->hash);
		}
	} else {
		// Not initialized the "web app" data

		// Initializing account
		$account = $context->get('account');

		if ($account) {
			// Initialized the account

			if (!empty($message)) {
				// Initialized the message

				// Initializing the contact data
				$contact = $message?->getContact();

				if (!empty($contact)) {
					// Initialized the contact data

					// Sanitizing received SIM-number (only numbers)
					$sanitized = preg_replace('/[^\d]/', '', $contact->getPhoneNumber());

					if (!empty($sanitized)) {
						// Sanitized receiver SIM-number

						// Writing receiver SIM-number into the account
						$account->receiver = ['sim' => (int) $sanitized] + ($account->receiver ?? []);

						// Deabstracting the language parameter
						$account->language = $account->language->name;

						if (document::update($account->__document())) {
							// Writed the account instance into the ArangoDB document

							/* 
								// Sending the message
								$context->sendMessage(
								<<<TXT
								✅ *SIM\-номер зарегистрирован:* $sanitized
								TXT,
								[
									'reply_markup' => [
										'remove_keyboard' => true
									]
								]
							)->then(function ($message) use ($context) {
								// Sended message

								// Sending the account parameters menu
								telegram::account_parameters($context);
							}); */

							// Sending the message
							$context->sendMessage(
								<<<TXT
								✅ *Номер телефона зарегистрирован:* $sanitized
								TXT,
								[
									'reply_markup' => [
										'remove_keyboard' => true
									]
								]
							)->then(function ($message) use ($context) {
								// Sended message

								// Sending the account parameters menu
								telegram::account_parameters($context);
							});
						} else {
							// Not writed the account instance into the ArangoDB document

							// Sending the message
							$context->sendMessage('⚠️ *Не удалось записать SIM\-номер*');
						}
					}
				}
			}
		}
	}

	unset($app);
});

$robot->onCommand('start', fn(context $context) => telegram::start($context));
$robot->onCommand('contacts', fn(context $context) => telegram::contacts($context));
$robot->onCommand('company', fn(context $context) => telegram::company($context));
$robot->onCommand('community', fn(context $context) => telegram::community($context));
$robot->onCommand('settings', fn(context $context) => telegram::settings($context));

$robot->onText('💬 Контакты', fn(context $context) => telegram::contacts($context));
$robot->onText('🏛️ О компании', fn(context $context) => telegram::company($context));
$robot->onText('🎯 Сообщество', fn(context $context) => telegram::community($context));
$robot->onText('⚙️  Настройки', fn(context $context) => telegram::settings($context));

$robot->onCbQueryData(['contacts'], fn(context $context) => telegram::contacts($context));
$robot->onCbQueryData(['company'], fn(context $context) => telegram::company($context));
$robot->onCbQueryData(['community'], fn(context $context) => telegram::community($context));
$robot->onCbQueryData(['settings'], fn(context $context) => telegram::settings($context));

$robot->onCbQueryData(['mail'], fn(context $context) => telegram::_mail($context));
$robot->onCbQueryData(['import_request'], fn(context $context) => telegram::import_request($context));

$robot->onCbQueryData(['order_commentary_request'], fn(context $context) => telegram::order_commentary_request($context));
$robot->onCbQueryData(['order'], fn(context $context) => telegram::order($context));

$robot->onCbQueryData(['tuning'], fn(context $context) => telegram::tuning($context));
$robot->onCbQueryData(['brands'], fn(context $context) => telegram::brands($context));

$robot->onCbQueryData(['cart_delivery'], fn(context $context) => telegram::cart_delivery($context));

$robot->onCbQueryData(['delivery_registration_destination_office'], fn(context $context) => telegram::delivery_registration_destination_office($context));
$robot->onCbQueryData(['delivery_registration_destination_door'], fn(context $context) => telegram::delivery_registration_destination_door($context));
$robot->onCbQueryData(['delivery_registration_sim_input'], fn(context $context) => telegram::delivery_registration_sim_input($context));
$robot->onCbQueryData(['delivery_registration_name_input'], fn(context $context) => telegram::delivery_registration_name_input($context));
$robot->onCbQueryData(['delivery_registration_address_input'], fn(context $context) => telegram::delivery_registration_address_input($context));
$robot->onCbQueryData(['delivery_registration'], fn(context $context) => telegram::delivery_registration($context));

$robot->onCbQueryData(['account_parameters_force'], fn(context $context) => telegram::account_parameters($context, force: true));

$robot->onCbQueryData(['receiver_sim_choose'], fn(context $context) => telegram::receiver_sim_choose($context));
$robot->onCbQueryData(['receiver_sim_request'], fn(context $context) => telegram::receiver_sim_request($context));
/* $robot->onCbQueryData(['receiver_sim_input'], fn(context $context) => telegram::receiver_sim_input($context)); */
$robot->onCbQueryData(['receiver_sim_write'], fn(context $context) => telegram::receiver_sim_write($context));

$robot->onCbQueryData(['receiver_name_choose'], fn(context $context) => telegram::receiver_name_choose($context));
$robot->onCbQueryData(['receiver_name_request'], fn(context $context) => telegram::receiver_name_request($context));
/* $robot->onCbQueryData(['receiver_name_input'], fn(context $context) => telegram::receiver_name_input($context)); */
$robot->onCbQueryData(['receiver_name_write'], fn(context $context) => telegram::receiver_name_write($context));

$robot->onCbQueryData(['receiver_destination_choose'], fn(context $context) => telegram::receiver_destination_choose($context));
$robot->onCbQueryData(['receiver_destination_office'], fn(context $context) => telegram::receiver_destination_office($context));
$robot->onCbQueryData(['receiver_destination_door'], fn(context $context) => telegram::receiver_destination_door($context));

$robot->onCbQueryData(['receiver_address_choose'], fn(context $context) => telegram::receiver_address_choose($context));
$robot->onCbQueryData(['receiver_address_request_office'], fn(context $context) => telegram::receiver_address_request($context, office: true));
$robot->onCbQueryData(['receiver_address_request_door'], fn(context $context) => telegram::receiver_address_request($context, office: false));
/* $robot->onCbQueryData(['receiver_address_input'], fn(context $context) => telegram::receiver_address_input($context)); */
$robot->onCbQueryData(['receiver_address_write'], fn(context $context) => telegram::receiver_address_write($context));

$robot->onException(function (Context $context, $exception) {
	var_dump($exception);
});

// Инициализация middleware с обработкой аккаунта
$robot->middleware([telegram::class, "account"]);

// Инициализация middleware с обработкой технических работ разных уровней
$robot->middleware([telegram::class, "suspension"]);

// Запуск чат-робота
$robot->run();
